﻿/*                         MIT License
                 Copyright (c) 2019 bitmap2tga by makeo

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Reflection;
using System.Text;
using TGASharpLib;

namespace png2tga
{
    class Program
    {
        #region options
        private const string HELP_FLAG = "h";
        private const string OVERRIDE_FLAG = "o";
        #endregion

        static void Main(string[] args)
        {
            try
            {
                string options = ParseOptions(args);

                if (options.Contains(HELP_FLAG))
                {
                    PrintHelp();
                }
                else if (args.Length < 2)
                {
                    PrintError("Wrong usage");
                    PrintHelp();
                }
                else
                {
                    if(ConvertToTga(args[0], args[1], options))
                    {
                        Console.WriteLine("Conversion successful");
                    }
                }
            }
            catch(Exception e)
            {
                Console.WriteLine(e.StackTrace);
                Console.Write("Unexpected error. Press any key to continue...");
                Console.ReadLine();
            }
        }

        private static string ParseOptions(string[] args)
        {
            StringBuilder options = new StringBuilder(3);

            for(int i = 0; i < args.Length; i++)
            {
                switch(args[i])
                {
                    case "-h":
                    case "--help":
                        options.Append(HELP_FLAG);
                        break;
                    case "-o":
                    case "--override":
                        options.Append(OVERRIDE_FLAG);
                        break;
                    default:
                        if(i >= 2)
                        {
                            PrintWarning("Unknown option " + args[i]);
                        }
                        break;
                }
            }

            return options.ToString();
        }

        private static bool ConvertToTga(string pngFile, string tgaFile, string options)
        {
            if(!File.Exists(pngFile))
            {
                PrintError("Bitmap file does not exist.");
                return false;
            }

            if(!options.Contains(OVERRIDE_FLAG) && File.Exists(tgaFile))
            {
                PrintError("TGA file does already exist.");
                return false;
            }

            if(!Directory.GetParent(tgaFile).Exists)
            {
                PrintError("TGA file directory does not exist.");
                return false;
            }

            Bitmap input = new Bitmap(new Bitmap(pngFile));
            input.RotateFlip(RotateFlipType.RotateNoneFlipY);
            Bitmap bmp = input.Clone(new Rectangle(0, 0, input.Width, input.Height), PixelFormat.Format24bppRgb);
            TGA tga = (TGA) bmp;
            tga.Header.ImageSpec.ImageDescriptor.ImageOrigin = TgaImgOrigin.BottomLeft;

            if (!tga.Save(tgaFile))
            {
                PrintError("Failed to save TGA");
                return false;
            }

            return true;
        }

        private static void PrintHelp()
        {
            var assembly = Assembly.GetExecutingAssembly();
            var resourceName = "bitmap2tga.help.txt";

            using (Stream stream = assembly.GetManifestResourceStream(resourceName))
            using (StreamReader reader = new StreamReader(stream))
            {
                Console.WriteLine(reader.ReadToEnd());
            }
        }

        private static void PrintWarning(string msg)
        {
            Console.WriteLine("[Warning] " + msg);
        }

        private static void PrintError(string msg)
        {
            Console.WriteLine("[Error] " + msg);
        }
    }
}
